@@ -160,7 +160,6 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 	
 	greeter_item_pam_set_user (buf);
-	greeter_item_ulist_enable ();
 	printf ("%c\n", STX);
 	fflush (stdout);
 	break;
@@ -250,10 +249,10 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 
 	tmp = ve_locale_to_utf8 (buf);
-	session = greeter_session_lookup (tmp);
+	session = gdm_session_lookup (tmp);
 	g_free (tmp);
 
-	if (greeter_save_session () == GTK_RESPONSE_CANCEL) {
+	if (gdm_get_save_session () == GTK_RESPONSE_CANCEL) {
 	     printf ("%c%s\n", STX, GDM_RESPONSE_CANCEL);
 	} else {
 	    tmp = ve_locale_from_utf8 (session);
@@ -279,7 +278,7 @@ greeter_ctrl_handler (GIOChannel *source,
     case GDM_SSESS:
         g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
-	if (greeter_save_session () == GTK_RESPONSE_YES)
+	if (gdm_get_save_session () == GTK_RESPONSE_YES)
 	  printf ("%cY\n", STX);
 	else
 	  printf ("%c\n", STX);
@@ -319,6 +318,8 @@ greeter_ctrl_handler (GIOChannel *source,
 	printf ("%c\n", STX);
 	fflush (stdout);
 	greeter_ignore_buttons (FALSE);
+	greeter_item_ulist_enable ();
+
 	break;
 
     case GDM_QUIT:
@@ -531,10 +532,10 @@ greeter_setup_items (void)
   greeter_item_timed_setup ();
   greeter_item_register_action_callback ("ok_button",
 					 greeter_ok_handler,
-					 window);
+					 (gpointer) window);
   greeter_item_register_action_callback ("cancel_button",
 					 greeter_cancel_handler,
-					 window);
+					 (gpointer) window);
   greeter_item_register_action_callback ("language_button",
 					 greeter_language_handler,
 					 NULL);
@@ -578,8 +579,8 @@ verify_gdm_version (void)
       gdm_wm_focus_new_windows (TRUE);
     
       msg =  g_strdup_printf (_("The greeter version (%s) does not match the daemon "
-                                "version.  "
-                                "You have probably just upgraded GDM.  "
+                                "version. "
+                                "You have probably just upgraded GDM. "
                                 "Please restart the GDM daemon or the computer."),
                               VERSION);
     
@@ -732,6 +733,9 @@ gdm_set_welcomemsg (void)
 static void
 gdm_read_config (void)
 {
+	/* Read config data in bulk */
+	gdmcomm_comm_bulk_start ();
+
 	gdmcomm_set_debug (gdm_config_get_bool (GDM_KEY_DEBUG));
 
 	/*
@@ -791,12 +795,15 @@ gdm_read_config (void)
 	gdm_config_get_string (GDM_KEY_PID_FILE);
 	gdm_config_get_string (GDM_KEY_PRE_FETCH_PROGRAM);
 
-	gdmcomm_comm_close ();
+	gdmcomm_comm_bulk_stop ();
 }
 
 static gboolean
 greeter_reread_config (int sig, gpointer data)
 {
+	/* Read config data in bulk */
+	gdmcomm_comm_bulk_start ();
+
 	if (gdm_config_reload_bool (GDM_KEY_DEBUG))
 		gdmcomm_set_debug (gdm_config_get_bool (GDM_KEY_DEBUG));
 
@@ -847,7 +854,7 @@ greeter_reread_config (int sig, gpointer data)
 		gdm_common_setup_cursor (GDK_WATCH);
 
 		gdm_wm_save_wm_order ();
-		gdmcomm_comm_close ();
+		gdmcomm_comm_bulk_stop ();
 
 		_exit (DISPLAY_RESTARTGREETER);
 	}
@@ -868,12 +875,12 @@ greeter_reread_config (int sig, gpointer data)
 		gdm_common_setup_cursor (GDK_WATCH);
 
 		gdm_wm_save_wm_order ();
-		gdmcomm_comm_close ();
+		gdmcomm_comm_bulk_stop ();
 
 		_exit (DISPLAY_RESTARTGREETER);
 	}
 
-	gdmcomm_comm_close ();
+	gdmcomm_comm_bulk_stop ();
 
 	return TRUE;
 }
@@ -1028,7 +1035,6 @@ main (int argc, char *argv[])
   GIOChannel *ctrlch;
   GError *error;
   GreeterItemInfo *root;
-  const gchar *theme_name;
   char *theme_file;
   char *theme_dir;
   gchar *gdm_graphical_theme;
@@ -1301,7 +1307,7 @@ main (int argc, char *argv[])
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
 
-      execl (EXPANDED_LIBEXECDIR "/gdmlogin", EXPANDED_LIBEXECDIR "/gdmlogin", NULL);
+      execl (LIBEXECDIR "/gdmlogin", LIBEXECDIR "/gdmlogin", NULL);
       execlp ("gdmlogin", "gdmlogin", NULL);
 
       dialog = ve_hig_dialog_new (NULL /* parent */,
@@ -1335,6 +1341,9 @@ main (int argc, char *argv[])
   gtk_window_move (GTK_WINDOW (window), gdm_wm_screen.x, gdm_wm_screen.y);
   gtk_widget_show_now (window);
 
+  greeter_item_ulist_unset_selected_user ();
+  greeter_item_ulist_enable ();
+
   /* can it ever happen that it'd be NULL here ??? */
   if G_UNLIKELY (window->window != NULL)
     {
