@@ -61,6 +61,14 @@ enum
 	GREETER_ULIST_ACTIVE_COLUMN
 };
 
+void
+greeter_item_ulist_unset_selected_user (void)
+{
+        GtkTreeSelection *selection;
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (user_list));
+        gtk_tree_selection_unselect_all (selection);
+}
+
 static void
 check_for_displays (void)
 {
@@ -325,8 +333,6 @@ greeter_item_ulist_setup (void)
 			force_no_tree_separators (user_list);
 
 			greeter_generate_userlist (user_list);
-			if ( ! DOING_GDM_DEVELOPMENT)
-				greeter_item_ulist_disable ();
 
 			/* Reset size of the widget canvas item so it
 			 * is the same size as the userlist.  This
